function spheroidGPS(problem)
close all; clc;

% this line must be commented before compiling
% clear all; path(pathdef);

if nargin == 0
    problem = 'invert';
elseif strcmpi(problem,'help')
disp('=========================================================================');
disp('spheroidGPS.m  v1.1 - M. Battaglia - mbattaglia@usgs.gov');
disp('This program inverts GPS measurements to find the best fit spherical source.'); 
disp('The inversion algorithm uses the MATLAB function fmincon (optimization toolbox)');
disp('    with a random grid search');
disp('The spheroid script is based on Yang et al (1988), JGR 93(B5), 4249-4257.');
disp(' ');
disp('Usage: spheroidGPS');
disp('       input_data_file.dat: input file with inversion parameters');
disp('       GPSdatafile.dat    : input file with GPS data');
disp('       GPSdatafile.rsl    : ouput file with inversion results');
disp('       GPSstatistics.jpg  : figure with inversion statistics');
disp('       GPSbestfit.jpg     : figure with profile plot of inversion results');
disp('       GPShorvrl.jpg      : figure with vector plot of inversion results');
disp('       GPSvertvel.jpg     : figure with vector plot of inversion results');
disp(' ');
disp('Examples: spheroidGPS help'); 
disp('          spheroidGPS')
disp(' ');
disp('*** THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND ***');
disp('=========================================================================');
return;
end;

%==========================================================================
% USGS Software Disclaimer 
% The software and related documentation were developed by the U.S. 
% Geological Survey (USGS) for use by the USGS in fulfilling its mission. 
% The software can be used, copied, modified, and distributed without any 
% fee or cost. Use of appropriate credit is requested. 
%
% The USGS provides no warranty, expressed or implied, as to the correctness 
% of the furnished software or the suitability for any purpose. The software 
% has been tested, but as with any complex software, there could be undetected 
% errors. Users who find errors are requested to report them to the USGS. 
% The USGS has limited resources to assist non-USGS users; however, we make 
% an attempt to fix reported problems and help whenever possible. 
%==========================================================================

% *** INITIALIZE PARAMETERS ===============================================
% fmisearch OPTIONS 
options = optimset('TolFun',1e-7,'TolX',1e-7,'MaxFunEvals',2000,'MaxIter',2000,'Algorithm','interior-point','Display','off');
% define figure size
scsz = get(0,'ScreenSize');
% =========================================================================


% =========================================================================
% *** READ INPUT PARAMETER FILE 
fid0 = fopen('input_data_file.dat','r');
inputfile = textscan(fid0,'%s',1,'HeaderLines',2);
output_file = strcat(char(inputfile{1}),'.rsl');                               
input_file = strcat(char(inputfile{1}),'.dat');
% random walk
Mrw = textscan(fid0,'%f %f',1,'HeaderLines',5);
yrw = Mrw{1}; srw = Mrw{2};
% elastic constants  
Mec = textscan(fid0,'%f %f',1,'HeaderLines',5);
lambda = Mec{1}; mu = Mec{2}; nu=0.5*lambda/(lambda+mu);                    % lambda, shear modulus, Poisson's ratio
% number of random grid searches
Maxk = textscan(fid0,'%f',1,'HeaderLines',3); 
maxk = Maxk{1};
% sphere radius
Ma = textscan(fid0,'%f',1,'HeaderLines',3); 
a = Ma{1};
% search radius
Msr = textscan(fid0,'%f',1,'HeaderLines',3); 
SR = Msr{1};

% *** READ DATA FILE  
fprintf(1,'------ LOAD DATA -------, loading %s\n',char(input_file));
fidinp = fopen(input_file,'r');
Mco = textscan(fidinp,'%q',1); comment = char(Mco{1});
Minp = textscan(fidinp,'%s %f %f %f %f %f %f %f %f','CommentStyle','%');
site = Minp{1}; xsite = Minp{2}; ysite = Minp{3}; 
east = Minp{4}; stdes = Minp{5}; north = Minp{6}; stdno = Minp{7}; 
up   = Minp{8}; stdup = Minp{9};

% get location of vent (if available)
k = 0;
for i=1:length(site)
    if strcmp(site(i),'VENT') == 1
        Xvent = xsite(i); Yvent = ysite(i);
    else
        k=k+1;
        bm(k) = site(i);
        x(k)  = xsite(i);
        y(k)  = ysite(i);
        u(k)  = east(i);
        du(k) = stdes(i);
        v(k)  = north(i);
        dv(k) = stdno(i);
        w(k)  = up(i);
        dw(k) = stdup(i);
    end;
end;
% =========================================================================

% *** PREALLOCATE VARIABLES ***********************************************
x0 = zeros(maxk,1); y0 = zeros(maxk,1); z0 = zeros(maxk,1); Ak = zeros(maxk,1);
thetak = zeros(maxk,1); phik = zeros(maxk,1);
bf_chi = zeros(maxk,1);
bf_x0 = zeros(maxk,1); bf_y0 = zeros(maxk,1); bf_z0 = zeros(maxk,1);
bf_A = zeros(maxk,1); bf_theta = zeros(maxk,1); bf_phi = zeros(maxk,1);
bf_ef = zeros(maxk,1); bf_P_G = zeros(maxk,1);
% *************************************************************************

% *** START PRINTING TO OUTPUT FILES **************************************
fidout = fopen(output_file,'w+');                                           % output data file fid
% *************************************************************************

% *************************************************************************
disp('------ DATA VECTOR -----');
data = [u v w]; data = data(:);                                             % deformation
sigma = [du dv dw]; sigma = sigma(:);                                       % uncertainty 

% *************************************************************************
disp('------ COVARIACE MATRIX -----');
SigWN = diag(sigma.^2);                                                     % diagonal covariance matrix, m^2
SigRW = srw^2*yrw*eye(size(SigWN));                                         % random walk noise in m^2, (0.5 mm/sqrt(yr))
SigH  = SigWN + SigRW;                                                      % covariance matrix, m^2
WH = SigH\eye(size(SigH));                                                  % weight for least square, 1/m^2 (equivalent to inv(SigH))
% *************************************************************************

% *************************************************************************
disp('------ START MIN SEARCH - UPLIFT -------');
    if mean(w) > 0
         minp = 0.0001; maxp = 0.1;
    else
        minp = -0.1; maxp = -0.0001;
    end;

  
h = waitbar(0,'Please wait ...');    
k = 1;  
while k <= maxk

    Xi = Xvent + normrnd(0,1E4); Yi = Yvent + normrnd(0,1E4);Zi = a + abs(normrnd(1E4,5E3));
    Ai = abs(0.4*(1+0.5*normrnd(0,1))); 
    thetai = abs(45*(1+0.3*normrnd(0,1))); 
    phii = abs(180*(1+0.5*normrnd(0,1)));

    [BF,fval,exitflag] = fmincon(@(VAR) X2v_spheroid(VAR,a,mu,nu,x,y,WH,data),...
                         [Xi,Yi,Zi,Ai,0.01*sign(mean(w)),thetai,phii],[],[],[],[],...
                         [Xvent-SR,Yvent-SR,a,0,minp,0,0],[Xvent+SR,Yvent+SR,2E4,1,maxp,90,360],...
                         [],options);
                     
    if exitflag > 0
        x0(k) = Xi; y0(k) = Yi; z0(k) = Zi; Ak(k) = Ai; thetak(k) = thetai; phik(k) = phii; 
        bf_ef(k)    = exitflag;
        bf_chi(k)   = fval;                                                 % best fit chi square
        bf_x0(k)    = BF(1);                                                % best fit source location (East) 
        bf_y0(k)    = BF(2);                                                % best fit source locationb (North)
        bf_z0(k)    = BF(3);                                                % best fit depth, m 
        bf_A(k)     = BF(4);                                                % best fit aspect ratio
        bf_P_G(k)   = BF(5);                                                % best fit pressure/shear modulus    
        bf_theta(k) = BF(6);                                                % best fit dip angle
        bf_phi(k)   = BF(7);                                                % best fit strike angle
        k = k+1;
    end; 
    waitbar(k/maxk);
end;
        bf_dV       = pi*bf_P_G.*a^3.*bf_A.^2.*(bf_A.^2-2*bf_A+4)/3;         % best fit volume, m^3    
close(h);

% determine best fit parameters
[chi2u,Iu] = min(bf_chi);
X0 = bf_x0(Iu);                                                             % source location (east) 
Y0 = bf_y0(Iu);                                                             % source location (north)
Z0 = bf_z0(Iu);                                                             % source location (depth)
A  = bf_A(Iu);                                                              % aspect ratio
dP_G = bf_P_G(Iu);                                                          % pressure/shear modulus
theta = bf_theta(Iu);                                                       % dip angle
phi = bf_phi(Iu);                                                           % strike angle
efu = bf_ef(Iu);                                                            % exit flag
dV = pi*dP_G*a^3*A^2*(A^2/3 -0.7*A +1.37);                                  % volume change, m^3

figure('Position',[50 50 0.8*scsz(3) 0.8*scsz(4)],'Name','STATISTICS: UPLIFT INVERSION');                         
ix = linspace(min(x0),max(x0),10);                                             
iy = linspace(min(y0),max(y0),10);
iz = linspace(min(z0),max(z0),10);
iA = linspace(min(Ak),max(Ak),10);
it = linspace(min(thetak),max(thetak),10);
ip = linspace(min(phik),max(phik),10);
subplot(2,8,1); hist(bf_ef,20); title(strcat('exitflag: ',sprintf(' %2.0f',efu)));
subplot(2,8,2); hist(bf_chi,5); title(strcat('\chi^2: ',sprintf(' %4.2f',chi2u)));
subplot(2,8,3); hist(x0,ix); title('x_0');
subplot(2,8,4); hist(y0,iy); title('y_0');
subplot(2,8,5); hist(z0,iz); title('z_0');
subplot(2,8,6); hist(Ak,iA); title('A');
subplot(2,8,7); hist(thetak,it); title('\theta');
subplot(2,8,8); hist(phik,ip); title('\phi');
subplot(2,8,9); hist(bf_P_G,length(iz)); title(strcat('bf \DeltaP/\mu: ',sprintf('%4.2e',dP_G)));
subplot(2,8,10); hist(bf_dV,length(iz)); title(strcat('bf \DeltaV: ',sprintf('%4.2e',dV)));
subplot(2,8,11); hist(bf_x0,ix); title(strcat('bf x_0: ',sprintf('%8.0f',X0)));
subplot(2,8,12); hist(bf_y0,iy); title(strcat('bf y_0: ',sprintf('%8.0f',Y0)));
subplot(2,8,13); hist(bf_z0,iz); title(strcat('bf z_0: ',sprintf('%6.0f',Z0)));
subplot(2,8,14); hist(bf_A,iA); title(strcat('bf A: ',sprintf(' %3.2f',A))); 
subplot(2,8,15); hist(bf_theta,it); title(strcat('bf \theta: ',sprintf(' %2.0f',theta))); 
subplot(2,8,16); hist(bf_A,ip); title(strcat('bf \phi: ',sprintf(' %3.0f',phi))); 

set(gcf,'PaperPositionMode','auto'); print -djpeg -r300 spheroidGPSstat; 
% *************************************************************************


[U V W] = yang(X0,Y0,Z0,a,A,dP_G,mu,nu,theta,phi,x,y,0);                    % best fit solution


% *************************************************************************
disp('------ PRINT OUTPUT FILE ------');
fprintf(fidout,'%s; data file: %s\n',comment,input_file);
fprintf(fidout,'Source parameters (SI units), pressure change is relative to mu\n');
fprintf(fidout,'X2v: %4.2e\n',chi2u);
fprintf(fidout,'X0 Y0 Z0 radius dP(-) dV\n');
fprintf(fidout,'%8.0f %8.0f %5.0f %4.0f %4.2e %4.2e\n',X0,Y0,Z0,a,dP_G,dV);
fprintf(fidout,'A theta phi \n');
fprintf(fidout,'%3.2f %3.0f %3.0f \n',A,theta,phi);
 bm = char(bm);
 fprintf(fidout,'Site x y E mE dE N mN dN W mW dW \n');
 for i=1:length(x)
     fprintf(fidout,'%-5s %-8.0f %-8.0f %-6.4f %-6.4f %-6.4f %-6.4f %-6.4f %-6.4f %-6.4f %-6.4f %-6.4f\n',...
                       bm(i,:),x(i),y(i),u(i),U(i),du(i),v(i),V(i),dv(i),w(i),W(i),dw(i));
 end;   
% *************************************************************************


disp('------ PLOT BEST FIT  PROFILE------')
R = sqrt((x-X0).^2 + (y-Y0).^2);
UR = sqrt(U.^2+V.^2);                                                       % best fit radial deformation
ur = sqrt(u.^2+v.^2); dur = sqrt((u.*du./ur).^2+(v.*dv./ur).^2);            % radial deformation and error

figure('Position',[10 scsz(4)/10 scsz(3)/2 scsz(4)/2],'Name','PROFILE');
subplot(1,2,1)
errorbar(0.001*R,ur,dur,'r+'); hold on;                                     % data
plot(0.001*R,UR,'sb');                                                      % model
xlabel('R (km)'); title('Horizontal deformation');
legend('data','model'); legend('Location','best'); legend('boxoff');
subplot(1,2,2)
errorbar(0.001*R,w,dw,'r+'); hold on;                                       % data
plot(0.001*R,W,'sb');                                                       % model
xlabel('R (km)'); title('Vertical deformation');
legend('data','model'); legend('Location','best'); legend('boxoff');

set(gcf,'PaperPositionMode','auto')
print -djpeg -r300 -zbuffer spheroidGPSbestfit; 



disp('------ PLOT GPS VELOCITIES  ------')
hscale = max(sqrt(u.^2+v.^2)); vscale = max(w);                             % horizontal and vertical scale
site = char(bm);                                                            % site labels

% plot with GPS horizontal velocities
figure('Position',[10 10 0.8*scsz(4) 0.8*scsz(4)],'Name','GPS');
hold on; 
plot(X0,Y0,'or','MarkerFaceColor','y');                                     % plot source location
plot(Xvent,Yvent,'or','MarkerSize',10,'MarkerFaceColor','w');               % plot crater location
plot(x,y,'s','MarkerSize',4); text(x*1.0005,y,site,'FontSize',4);           % plot GPS sites
quiver(x,y,u/hscale,v/hscale,'r','LineWidth',2);                            % vector plot - data
quiver(x,y,U/hscale,V/hscale,'b');                                          % vector plot - model
title('Horizontal deformation [m] - red arrows: data');
set(gcf,'PaperPositionMode','auto')
print -djpeg -r300 -zbuffer spheroidGPShorivel; 

% plot with GPS vertical velocities
figure('Position',[10 10 0.8*scsz(4) 0.8*scsz(4)],'Name','VERT DEF');
hold on; 
plot(X0,Y0,'or','MarkerFaceColor','y');                                     % plot source location
plot(Xvent,Yvent,'or','MarkerSize',10,'MarkerFaceColor','w');               % plot crater location
plot(x,y,'s','MarkerSize',4); text(x*1.0005,y,site,'FontSize',4);           % plot GPS sites
quiver(x,y,zeros(size(W)),w/vscale,'r','LineWidth',2);                      % vector plot - data
quiver(x,y,zeros(size(W)),W/vscale,'b');                                    % vector plot - model
title('Vertical deformation [m] - red arrows: data');
set(gcf,'PaperPositionMode','auto')
print -djpeg -r300 -zbuffer spheroidGPSvertvel; 

% *************************************************************************
fclose('all'); disp('That''s all Folks!');