function sphereTILT(problem)
close all; clc;

% this line must be commented before compiling
%clear all; problem = 'inflation'; path(pathdef);

if strcmpi(problem,'help')
disp('=========================================================================');
disp('sphereTILT.m  v1.2 (Apr 10, 2012) - M. Battaglia - mbattaglia@usgs.gov');
disp('This program inverts TILT measurements to find the best fit spherical source.'); 
disp('The inversion algorithm uses the MATLAB function fmincon (optimization toolbox)');
disp('    with a random grid search');
disp('The sphere script is based on McTigue (1987), JGR 92, 12931-12940.');
disp(' ');
disp('Usage: sphereTILT problem');
disp('       problem is a string = "inflation" or "deflation"'); 
disp('       TILTdatafile.dat   : input file with TILT data');
disp('       TILTdatafile.rsl   : ouput file with inversion results');
disp('       TILTstatistics.jpg : figure with inversion statistics');
disp('       TILTvectorplot.jpg : figure with vector plot of inversion results');
disp(' ');
disp('Examples: sphereTILT help'); 
disp('          sphereTILT inflation')
disp(' ');
disp('*** THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND ***');
disp('=========================================================================');
return;
end;


% *** INITIALIZE PARAMETERS ===============================================
% fmisearch OPTIONS 
options = optimset('TolFun',1e-10,'TolX',1e-10,'MaxFunEvals',2000,'MaxIter',2000,'Algorithm','interior-point','Display','off');
% define figure size
scsz = get(0,'ScreenSize');
% =========================================================================


% =========================================================================
% *** READ INPUT PARAMETER FILE 
fid0 = fopen('input_data_file.dat','r');
inputfile = textscan(fid0,'%s',1,'HeaderLines',2);
output_file = strcat(char(inputfile{1}),'.rsl');                               
input_file = strcat(char(inputfile{1}),'.dat');
% random walk
Mrw = textscan(fid0,'%f %f',1,'HeaderLines',5);
yrw = Mrw{1}; srw = Mrw{2};
% elastic constants  
Mec = textscan(fid0,'%f %f',1,'HeaderLines',5);
lambda = Mec{1}; mu = Mec{2}; nu=0.5*lambda/(lambda+mu);                    % lambda, shear modulus, Poisson's ratio
% number of random grid searches
Maxk = textscan(fid0,'%f',1,'HeaderLines',3); 
maxk = Maxk{1};
% sphere radius
Ma = textscan(fid0,'%f',1,'HeaderLines',3); 
a = Ma{1};
% search radius
Msr = textscan(fid0,'%f',1,'HeaderLines',3); 
SR = Msr{1};

% *** READ DATA FILE  
fprintf(1,'------ LOAD DATA -------, loading %s\n',char(input_file));
fidinp = fopen(input_file,'r');
Mco = textscan(fidinp,'%q',1); comment = char(Mco{1});
Minp = textscan(fidinp,'%s %f %f %f %f %f %f','CommentStyle','%');
site = Minp{1}; xsite = Minp{2}; ysite = Minp{3}; 
east = Minp{4}; stdes = Minp{5}; north = Minp{6}; stdno = Minp{7};

% get location of vent (if available)
k = 0;
for i=1:length(site)
    if strcmp(site(i),'VENT') == 1
        Xvent = xsite(i); Yvent = ysite(i);
    else
        k=k+1;
        bm(k) = site(i);
        x(k)  = xsite(i);
        y(k)  = ysite(i);
        e(k)  = east(i);                                                    % east component of tilt
        de(k) = stdes(i);                                                   % uncertainty of east component of tilt
        n(k)  = north(i);                                                   % nort component of tilt
        dn(k) = stdno(i);                                                   % uncertainty of north component of tilt
    end;
end;
% =========================================================================


% allocate variables (this variables change size during the inversion)
x0 = zeros(maxk,1); y0 = zeros(maxk,1); z0 = zeros(maxk,1); bf_chi = zeros(maxk,1);
bf_x0 = zeros(maxk,1); bf_y0 = zeros(maxk,1); bf_z0 = zeros(maxk,1);
bf_ef = zeros(maxk,1); bf_P_G = zeros(maxk,1); bf_dV = zeros(maxk,1);

% =========================== INVERSION ===================================

disp('------ DATA VECTOR -----');
%data = sqrt(e.^2 + n.^2);                                                   % tilt
%sigma2 = ((e.*de).^2 + (n.*dn).^2)./data.^2;                                % uncertainty

data = [e, n]';                                                             % tilt (components)
sigma = [de, dn]';                                                          % uncertainties
sigma2 = sigma.^2;

disp('------ COVARIACE MATRIX -----');
SigH = diag(sigma2);                                                        % diagonal covariance matrix,
WH = SigH\eye(size(SigH));                                                  % weight for least square 

disp('------ START MIN SEARCH - UPLIFT -------');
    if strcmpi(problem,'inflation') == 1 
         minp = 0.0001; maxp = 0.5; signP = 1;
    else
        minp = -0.5; maxp = -0.0001; signP = -1;
    end;

k = 1;
h = waitbar(0,'Please wait ...');
while k <= maxk

   Xi = Xvent + normrnd(0,1E4); Yi = Yvent + normrnd(0,1E4);
   Zi = a + abs(normrnd(1E4,5E3));
   [BF,fval,exitflag] = fmincon(@(VAR) X2v_sphereTILT(VAR,a,nu,x,y,WH,data),...
                        [Xi,Yi,Zi,0.01*signP],[],[],[],[],...
                        [Xvent-SR,Yvent-SR,a,minp],[Xvent+SR,Yvent+SR,2E4,maxp],[],options);

    if exitflag > 0
        x0(k) = Xi; y0(k) = Yi; z0(k) = Zi; 
        bf_ef(k)  = exitflag;
        bf_chi(k) = fval;                                                   % best fit chi square
        bf_x0(k)  = BF(1);                                                  % best fit source location (East) 
        bf_y0(k)  = BF(2);                                                  % best fit source locationb (North)
        bf_z0(k)  = BF(3);                                                  % best fit depth, m 
        bf_P_G(k) = BF(4);                                                  % best fir pressure/shear modulus    
        bf_dV(k) = bf_P_G(k)*a^3*pi*(1+(a/bf_z0(k))^4);                     % volume change according to Battaglia et al, 2011 - m^3
        k = k+1;
    end; 
    
    waitbar(k/maxk);
end;
close(h);

% determine best fit parameters
[chi2u,Iu] = min(bf_chi);
X0 = bf_x0(Iu);                                                             % source location (east) 
Y0 = bf_y0(Iu);                                                             % source location (north)
Z0 = bf_z0(Iu);                                                             % source location (depth)
efu = bf_ef(Iu);                                                            % exit flag
dP_G = bf_P_G(Iu);                                                          % pressure/shear modulus
dV = bf_dV(Iu);                                                             % volume change, m^3
% *******
[~, ~, ~, DWDX DWDY] = mctigue2D(X0,Y0,Z0,dP_G,a,nu,x,y);                   % best fit solution
% *******

disp('------ PLOT INVERSION STATISTICS ------');
figure('Position',[50 50 0.8*scsz(3) 0.8*scsz(4)],'Name','STATISTICS');                         
xi = linspace(min(x0),max(x0),10);                                             
yi = linspace(min(y0),max(y0),10);
zi = linspace(min(z0),max(z0),10);
subplot(2,5,1); hist(bf_ef,10); title(strcat('exitflag: ',sprintf('%2.0f',efu)));
subplot(2,5,2); hist(bf_chi,10); title(strcat('\chi^2: ',sprintf(' %4.2f',chi2u)));
subplot(2,5,3); hist(x0,xi); title('x_0');
subplot(2,5,4); hist(y0,yi); title('y_0');
subplot(2,5,5); hist(z0,zi); title('z_0');
subplot(2,5,6); hist(bf_P_G,length(zi)); title(strcat('bf \DeltaP/\mu: ',sprintf(' %4.2e',dP_G)));
subplot(2,5,7); hist(bf_dV,length(zi)); title(strcat('bf \DeltaV: ',sprintf(' %4.2e',dV))); 
subplot(2,5,8); hist(bf_x0,xi); title(strcat('bf x_0: ',sprintf('%8.0f',X0)));
subplot(2,5,9); hist(bf_y0,yi); title(strcat('bf y_0: ',sprintf('%8.0f',Y0)));
subplot(2,5,10); hist(bf_z0,zi); title(strcat('bf z_0: ',sprintf('%6.0f',Z0)));

set(gcf,'PaperPositionMode','auto'); print -djpeg -r300 TILTstatistics; 

% =========================================================================


disp('------ PRINT OUTPUT FILE ------');
fidout = fopen(output_file,'w+');                                           % output data file fid
fprintf(fidout,'Comment: %s\n',char(comment));
fprintf(fidout,'Source parameters; input file: %s\n',input_file);
fprintf(fidout,'X2v: %4.2e\n',chi2u);
fprintf(fidout,'X0 Y0 Z0 radius dP(-) dV\n');
fprintf(fidout,'%8.0f %8.0f %6.0f %4.0f %4.3e %4.3e\n',X0,Y0,Z0,a,dP_G,dV);
 bm = char(bm);
 fprintf(fidout,'Site x y E mE dE N mN dN\n');
 for i=1:length(x)
     fprintf(fidout,'%-5s %-8.0f %-8.0f %-8.6f %-8.6f %-8.6f %-8.6f %-8.6f %-8.6f\n',...
                       bm(i,:),x(i),y(i),e(i),DWDX(i),de(i),n(i),DWDY(i),dn(i));
 end;    


disp('------ PLOT TILT VECTORS  ------')
hscale = max(sqrt(e.^2+n.^2));                                              % vector scale

% plot TILT
figure('Position',[10 10 0.8*scsz(4) 0.8*scsz(4)],'Name','TILT');
hold on; 
plot(X0,Y0,'or','MarkerFaceColor','y');                                     % plot source location
plot(Xvent,Yvent,'or','MarkerSize',10,'MarkerFaceColor','w');               % plot crater location
plot(x,y,'s','MarkerSize',4); text(x*1.0005,y,char(bm),'FontSize',6);       % plot TILT sites
quiver(x,y,e/hscale,n/hscale,'r','LineWidth',2);                            % vector plot - data
quiver(x,y,DWDX/hscale,DWDY/hscale,'b');                                    % vector plot - model
title('Tilt vectors (red arrows: data)');
set(gcf,'PaperPositionMode','auto')
print -djpeg -r300 -zbuffer TILTvectorplot; 

% *************************************************************************

fclose('all'); disp('That''s all Folks!');
