function spheroidTILTv(problem)
close all; clc;

% this line must be commented before compiling (used for debug)
% clear all; path(pathdef);
% problem = 'inflation';

if nargin == 0
    problem = 'invert';
elseif strcmpi(problem,'help')
disp('=========================================================================');
disp('spheroidTILTv.m  v1.1 (Apr 10, 2012) - M. Battaglia - mbattaglia@usgs.gov');
disp('This program inverts TILT measurements to find the best fit vertical'); 
disp('    spheroidal source.'); 
disp('The inversion algorithm uses the MATLAB function fmincon (optimization toolbox)');
disp('    with a random grid search');
disp('The spheroid script is based on Yang et al (1988), JGR 93(B5), 4249-4257.');
disp(' ');
disp('Usage: spheroidTILT problem');
disp('       problem is a string = "inflation" or "deflation"'); 
disp('       input_data_file.dat: input file with inversion parameters');
disp('       TILTdatafile.dat    : input file with TILT data');
disp('       TILTdatafile.rsl    : ouput file with inversion results');
disp('       TILTstatistics.jpg : figure with inversion statistics');
disp('       TILTvectorplot.jpg : figure with vector plot of inversion results');
disp(' ');
disp('Examples: spheroidTILTv help'); 
disp('          spheroidTILTv inflation')
disp(' ');
disp('*** THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND ***');
disp('=========================================================================');
return;
end;

% *** INITIALIZE PARAMETERS ===============================================
% fmisearch OPTIONS 
options = optimset('TolFun',1e-7,'TolX',1e-7,'MaxFunEvals',2000,'MaxIter',2000,'Algorithm','interior-point','Display','off');
% define figure size
scsz = get(0,'ScreenSize');
% =========================================================================


% =========================================================================
% *** READ INPUT PARAMETER FILE 
fid0 = fopen('input_data_file.dat','r');
inputfile = textscan(fid0,'%s',1,'HeaderLines',2);
output_file = strcat(char(inputfile{1}),'.rsl');                               
input_file = strcat(char(inputfile{1}),'.dat');
% random walk
Mrw = textscan(fid0,'%f %f',1,'HeaderLines',5);
yrw = Mrw{1}; srw = Mrw{2};
% elastic constants  
Mec = textscan(fid0,'%f %f',1,'HeaderLines',5);
lambda = Mec{1}; mu = Mec{2}; nu=0.5*lambda/(lambda+mu);                    % lambda, shear modulus, Poisson's ratio
% number of random grid searches
Maxk = textscan(fid0,'%f',1,'HeaderLines',3); 
maxk = Maxk{1};
% sphere radius
Ma = textscan(fid0,'%f',1,'HeaderLines',3); 
a = Ma{1};
% search radius
Msr = textscan(fid0,'%f',1,'HeaderLines',3); 
SR = Msr{1};

% *** READ DATA FILE  
fprintf(1,'------ LOAD DATA -------, loading %s\n',char(input_file));
fidinp = fopen(input_file,'r');
Mco = textscan(fidinp,'%q',1); comment = char(Mco{1});
Minp = textscan(fidinp,'%s %f %f %f %f %f %f','CommentStyle','%');
site = Minp{1}; xsite = Minp{2}; ysite = Minp{3}; 
east = Minp{4}; stdes = Minp{5}; north = Minp{6}; stdno = Minp{7};

% get location of vent (if available)
k = 0;
for i=1:length(site)
    if strcmp(site(i),'VENT') == 1
        Xvent = xsite(i); Yvent = ysite(i);
    else
        k=k+1;
        bm(k) = site(i);
        x(k)  = xsite(i);
        y(k)  = ysite(i);
        e(k)  = east(i);                                                    % east component of tilt
        de(k) = stdes(i);                                                   % uncertainty of east component of tilt
        n(k)  = north(i);                                                   % nort component of tilt
        dn(k) = stdno(i);                                                   % uncertainty of north component of tilt
    end;
end;
% =========================================================================


% *** PREALLOCATE VARIABLES ***********************************************
x0 = zeros(maxk,1); y0 = zeros(maxk,1); z0 = zeros(maxk,1); Ak = zeros(maxk,1);
thetak = zeros(maxk,1); phik = zeros(maxk,1);
bf_chi = zeros(maxk,1);
bf_x0 = zeros(maxk,1); bf_y0 = zeros(maxk,1); bf_z0 = zeros(maxk,1);
bf_A = zeros(maxk,1); bf_theta = zeros(maxk,1); bf_phi = zeros(maxk,1);
bf_ef = zeros(maxk,1); bf_P_G = zeros(maxk,1);
% *************************************************************************


% *** START PRINTING TO OUTPUT FILES **************************************
fidout = fopen(output_file,'w+');                                           % output data file fid
% *************************************************************************


% =========================== INVERSION ===================================

disp('------ DATA VECTOR -----');
%data = sqrt(e.^2 + n.^2);                                                   % tilt
%sigma2 = ((e.*de).^2 + (n.*dn).^2)./data.^2;                                % uncertainty

data = [e, n]';                                                             % tilt (components)
sigma = [de, dn]';                                                          % uncertainties
sigma2 = sigma.^2;
 
disp('------ COVARIACE MATRIX -----');
SigH = diag(sigma2);                                                        % diagonal covariance matrix,
WH = SigH\eye(size(SigH));                                                  % weight for least square 


disp('------ START MIN SEARCH - TILT -------');

    if strcmpi(problem,'inflation') == 1 
         minp = 0.0001; maxp = 0.5; signP = 1;
    else
        minp = -0.5; maxp = -0.0001; signP = -1;
    end;
  
h = waitbar(0,'Please wait ...');    
k = 1;  
while k <= maxk

    Xi = Xvent + normrnd(0,1E4); Yi = Yvent + normrnd(0,1E4);Zi = a + abs(normrnd(1E4,5E3));
    Ai = abs(0.4*(1+0.5*normrnd(0,1))); 

    [BF,fval,exitflag] = fmincon(@(VAR) X2v_spheroidTILTv(VAR,a,mu,nu,x,y,WH,data),...
                         [Xi,Yi,Zi,Ai,0.01*signP],[],[],[],[],...
                         [Xvent-SR,Yvent-SR,a,0,minp],[Xvent+SR,Yvent+SR,2E4,1,maxp],...
                         [],options);
                     
    if exitflag > 0
        x0(k) = Xi; y0(k) = Yi; z0(k) = Zi; Ak(k) = Ai; 
        bf_ef(k)    = exitflag;
        bf_chi(k)   = fval;                                                 % best fit chi square
        bf_x0(k)    = BF(1);                                                % best fit source location (East) 
        bf_y0(k)    = BF(2);                                                % best fit source locationb (North)
        bf_z0(k)    = BF(3);                                                % best fit depth, m 
        bf_A(k)     = BF(4);                                                % best fit aspect ratio
        bf_P_G(k)   = BF(5);                                                % best fit pressure/shear modulus    
        k = k+1;
    end; 
    waitbar(k/maxk);
end;
        bf_dV       = pi*bf_P_G.*a^3.*bf_A.^2.*(bf_A.^2-2*bf_A+4)/3;         % best fit volume, m^3    
close(h);

% determine best fit parameters
[chi2u,Iu] = min(bf_chi);
X0 = bf_x0(Iu);                                                             % source location (east) 
Y0 = bf_y0(Iu);                                                             % source location (north)
Z0 = bf_z0(Iu);                                                             % source location (depth)
A  = bf_A(Iu);                                                              % aspect ratio
dP_G = bf_P_G(Iu);                                                          % pressure/shear modulus
theta = 90;                                                                 % dip angle
phi = 0;                                                                    % strike angle
efu = bf_ef(Iu);                                                            % exit flag
dV = pi*dP_G*a^3*A^2*(A^2/3 -0.7*A +1.37);                                  % volume change, m^3


[~, ~, ~, DWDX DWDY] = yang(X0,Y0,Z0,a,A,dP_G,mu,nu,theta,phi,x,y,0);       % best fit solution
% *************************************************************************


disp('------ PLOT INVERSION STATISTICS  ------')
figure('Position',[50 50 0.8*scsz(3) 0.8*scsz(4)],'Name','STATISTICS: UPLIFT INVERSION');                         
ix = linspace(min(x0),max(x0),10);                                             
iy = linspace(min(y0),max(y0),10);
iz = linspace(min(z0),max(z0),10);
iA = linspace(min(Ak),max(Ak),10);
subplot(2,6,1); hist(bf_ef,20); title(strcat('exitflag: ',sprintf(' %2.0f',efu)));
subplot(2,6,2); hist(bf_chi,5); title(strcat('\chi^2: ',sprintf(' %4.2f',chi2u)));
subplot(2,6,3); hist(x0,ix); title('x_0');
subplot(2,6,4); hist(y0,iy); title('y_0');
subplot(2,6,5); hist(z0,iz); title('z_0');
subplot(2,6,6); hist(Ak,iA); title('A');
subplot(2,6,7); hist(bf_P_G,length(iz)); title(strcat('bf \DeltaP/\mu: ',sprintf('%4.2e',dP_G)));
subplot(2,6,8); hist(bf_dV,length(iz)); title(strcat('bf \DeltaV: ',sprintf('%4.2e',dV)));
subplot(2,6,9); hist(bf_x0,ix); title(strcat('bf x_0: ',sprintf('%8.0f',X0)));
subplot(2,6,10); hist(bf_y0,iy); title(strcat('bf y_0: ',sprintf('%8.0f',Y0)));
subplot(2,6,11); hist(bf_z0,iz); title(strcat('bf z_0: ',sprintf('%6.0f',Z0)));
subplot(2,6,12); hist(bf_A,iA); title(strcat('bf A: ',sprintf(' %3.2f',A))); 

set(gcf,'PaperPositionMode','auto'); print -djpeg -r300 spheroidGPSstat; 


disp('------ PLOT TILT VECTORS  ------')
hscale = max(sqrt(e.^2+n.^2));                                              % vector scale

% plot TILT
figure('Position',[10 10 0.8*scsz(4) 0.8*scsz(4)],'Name','TILT');
hold on; 
plot(X0,Y0,'or','MarkerFaceColor','y');                                     % plot source location
plot(Xvent,Yvent,'or','MarkerSize',10,'MarkerFaceColor','w');               % plot crater location
plot(x,y,'s','MarkerSize',4); text(x*1.0005,y,char(bm),'FontSize',6);       % plot TILT sites
quiver(x,y,e/hscale,n/hscale,'r','LineWidth',2);                            % vector plot - data
quiver(x,y,DWDX/hscale,DWDY/hscale,'b');                                    % vector plot - model
title('Tilt vectors (red arrows: data)');
set(gcf,'PaperPositionMode','auto')
print -djpeg -r300 -zbuffer TILTvectorplot; 
% *************************************************************************


disp('------ PRINT OUTPUT FILE ------');
fprintf(fidout,'%s; data file: %s\n',comment,input_file);
fprintf(fidout,'Source parameters (SI units), pressure change is relative to mu\n');
fprintf(fidout,'X2v: %4.2e\n',chi2u);
fprintf(fidout,'X0 Y0 Z0 radius dP(-) dV\n');
fprintf(fidout,'%8.0f %8.0f %5.0f %4.0f %4.2e %4.2e\n',X0,Y0,Z0,a,dP_G,dV);
fprintf(fidout,'A theta phi \n');
fprintf(fidout,'%3.2f %3.0f %3.0f \n',A,theta,phi);
 bm = char(bm);
fprintf(fidout,'Site x y E mE dE N mN dN\n');
 for i=1:length(x)
     fprintf(fidout,'%-5s %-8.0f %-8.0f %-8.6f %-8.6f %-8.6f %-8.6f %-8.6f %-8.6f\n',...
                       bm(i,:),x(i),y(i),e(i),DWDX(i),de(i),n(i),DWDY(i),dn(i));
 end;    
% *************************************************************************

fclose('all'); disp('That''s all Folks!');